/*
 * To change this license header, choose License Headers in Project Properties.
 * To change this template file, choose Tools | Templates
 * and open the template in the editor.
 */
package moe.nekojimi.musicsearcher.providers;

import com.amihaiemil.eoyaml.YamlMapping;
import java.util.List;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.ForkJoinPool;
import java.util.concurrent.TimeUnit;
import java.util.concurrent.TimeoutException;
import moe.nekojimi.musicsearcher.Result;

/**
 *
 * @author jim
 */
public abstract class Searcher 
{
    final String name;
    private final ForkJoinPool executor;

    public Searcher(String name) 
    {
        this.name = name;
        this.executor = new ForkJoinPool();
    }
    
    public Searcher(YamlMapping yaml)
    {
        this(yaml.string("name"));
        assert yaml.string("type").equals(this.getClass().getSimpleName());
    }

    public String getName() 
    {
        return name;
    }
    
    public List<Result> searchAndWait(String query) throws InterruptedException, ExecutionException
    {
        try 
        {
            return searchAndWait(query, Long.MAX_VALUE, TimeUnit.DAYS);
        } catch (TimeoutException ex) 
        {
            throw new RuntimeException("This should never happen!",ex);
        }
    }
    
    public List<Result> searchAndWait(String query, long limit, TimeUnit unit) throws InterruptedException, ExecutionException, TimeoutException
    {
        return search(query).get(limit, unit);
    }
    
    public CompletableFuture<List<Result>> search(String query)
    {
        CompletableFuture<List<Result>> future = new CompletableFuture<>();
        future.completeAsync(()->doSearch(query), executor);
        return future;
    }

    protected abstract List<Result> doSearch(String query);

    @Override
    public String toString() {
        return this.getClass().getSimpleName() + "{" + "name=" + name + '}';
    }
    
    
}
